unit SharedObjectManagerClassUnit;

interface

uses Classes, SysUtils;

type
  TManObject = class;
  TManClass = class of TManObject;

  TSHOManager = class(TObject)
  private
    HBases: TStringList;
    HBaseClass: TManClass;
    procedure DelObj(const Ind: integer);
    procedure RemObj(const Obj: TManObject);
    procedure AddObj(const Key: string; const Obj: TManObject);
  protected
    function GetCount: integer;
  public
    property Count: integer read GetCount;
    function Objects(const Key: string; const CreateIfNewKey: boolean = false): TManObject; overload; 
    function Objects(const Index: integer): TManObject; overload;
    function IndexOf(const Key: string): integer;
    function Exists(const Key: string): boolean;
    procedure Delete(const Key: string);
    constructor Create(const BaseClass: TManClass);
    destructor Destroy; override;
  end;

  TManObject = class(TObject)
    private
      FshoKey: string;
      Fsho: TSHOManager;
    public
      property Key: string read FshoKey;
      constructor Create; overload; virtual;
      constructor Create(const Key: string; const SHO: TSHOManager); overload; virtual; 
      destructor Destroy; override;
    end;

implementation


// Implementation TManObject ============================================================================

constructor TManObject.Create;
begin
raise Exception.Create('It is not allowed to instantiate this class directly.');
end;

constructor TManObject.Create(const Key: string; const SHO: TSHOManager);
begin
inherited Create;
Fsho:=SHO;
FshoKey:=Key;
if Assigned(Fsho) then Fsho.AddObj(FshoKey,self);
end;

destructor TManObject.Destroy;
begin
if Assigned(Fsho) then Fsho.RemObj(self);
inherited Destroy;
end;

// Implementation TSHOManager ============================================================================

procedure TSHOManager.DelObj(const Ind: integer);
begin
HBases.Objects[Ind].Free;
HBases.Delete(Ind);
end;

procedure TSHOManager.RemObj(const Obj: TManObject);
var Ind: integer;
begin
Ind:=HBases.IndexOfObject(Obj);
if Ind>=0 then HBases.Delete(Ind);
end;

procedure TSHOManager.AddObj(const Key: string; const Obj: TManObject);
begin
if HBases.IndexOf(Key)<0
then HBases.AddObject(Key,Obj)
else raise Exception.Create('Cannot and object, key not unique');
end;

function TSHOManager.Objects(const Key: string; const CreateIfNewKey: boolean): TManObject;
var Ind: integer;
begin
Ind:=HBases.IndexOf(Key);
if Ind>=0
then Result:=HBases.Objects[Ind] as TManObject
else if CreateIfNewKey then Result:=HBaseClass.Create(Key,self)
else Result:=nil;
end;

function TSHOManager.Objects(const Index: integer): TManObject;
begin
Result:=HBases.Objects[Index] as TManObject
end;

function TSHOManager.IndexOf(const Key: string): integer;
begin
Result:=HBases.IndexOf(Key);
end;

function TSHOManager.Exists(const Key: string): boolean;
begin
Result:=HBases.IndexOf(Key)>=0;
end;

function TSHOManager.GetCount: integer;
begin
Result:=HBases.Count;
end;

procedure TSHOManager.Delete(const Key: string);
var Ind: integer;
begin
Ind:=HBases.IndexOf(Key);
if Ind>=0 then DelObj(Ind);
end;

constructor TSHOManager.Create(const BaseClass: TManClass);
begin
inherited Create;
HBaseClass:=BaseClass;
HBases:=TStringList.Create;
HBases.Sorted:=true;
HBases.Duplicates:=dupIgnore;
end;

destructor TSHOManager.Destroy;
var i: integer;
begin
for i:=0 to HBases.Count-1 do
  begin
  Objects(i).Fsho:=nil;
  Objects(i).Free;
  end;
HBases.Free;
inherited;
end;


end.

